﻿
using System.Windows;
using System.Windows.Media;

namespace FontDialog
{
    public static class Extensions
    {
        public static FontData ToFontData(this LogFontStruct logFont)
        {
            var result = new FontData();

            result.FontFamily = new FontFamily(logFont.lfFaceName);
            result.FontStyle = logFont.lfItalic != 0 ? FontStyles.Italic : FontStyles.Normal;

            // font weight as seen here: http://msdn.microsoft.com/en-us/library/ms602645.aspx
            var fontWeight = logFont.lfWeight;
            if (fontWeight == 100)
            {
                result.FontWeight = FontWeights.Thin;
            }
            else if (fontWeight == 200)
            {
                result.FontWeight = FontWeights.ExtraLight;
            }
            else if (fontWeight == 300)
            {
                result.FontWeight = FontWeights.Light;
            }
            else if (fontWeight == 400)
            {
                result.FontWeight = FontWeights.Normal;
            }
            else if (fontWeight == 500)
            {
                result.FontWeight = FontWeights.Medium;
            }
            else if (fontWeight == 600)
            {
                result.FontWeight = FontWeights.SemiBold;
            }
            else if (fontWeight == 700)
            {
                result.FontWeight = FontWeights.Bold;
            }
            else if (fontWeight == 800)
            {
                result.FontWeight = FontWeights.ExtraBold;
            }
            else if (fontWeight == 900)
            {
                result.FontWeight = FontWeights.Black;
            }
            else if (fontWeight == 950)
            {
                result.FontWeight = FontWeights.ExtraBlack;
            }
            else
            {
                result.FontWeight = FontWeights.Normal;
            }

            // size should be calculated as:
            // pointSize = MulDiv(-lfHeight, 72, GetDeviceCaps(hDC, LogPixelsY);
            // we assume:
            result.FontSize = -logFont.lfHeight * 72.0 / 96.0; // :>

            return result;
        }
    }
}
