﻿using System;
using System.Runtime.InteropServices;

namespace FontDialog
{
    public static class FontInteropHelper
    {
        private const int CF_SCREENFONTS = 0x00000001;

        /// <summary>
        /// Dokumentation can be found here: http://msdn.microsoft.com/en-us/library/ms646914(v=VS.85).aspx
        /// </summary>
        /// <param name="lpcf"></param>
        /// <returns></returns>
        [DllImport("comdlg32.dll", CharSet = CharSet.Auto, EntryPoint = "ChooseFont")]
        private extern static bool ChooseFont(IntPtr lpcf);

        public static FontData ShowFontDialog()
        {
            GCHandle lfBufferHandle = default(GCHandle);
            GCHandle cfBufferHandle = default(GCHandle);

            try
            {
                LogFontStruct lf = new LogFontStruct();

                // The following way of allocating memory is not available in Silverlight,
                // so we use GCHandle.Alloc instead
                //IntPtr lplf = AllocHGlobal(Marshal.SizeOf(lf));
                lfBufferHandle = GCHandle.Alloc(new byte[Marshal.SizeOf(lf)], GCHandleType.Pinned);
                IntPtr ptrLf = lfBufferHandle.AddrOfPinnedObject();
                
                // copy over
                Marshal.StructureToPtr(lf, ptrLf, false);

                // create new structure
                ChooseFontStruct cf = new ChooseFontStruct();
                cf.lStructSize = Marshal.SizeOf(typeof(ChooseFontStruct));
                cf.Flags = CF_SCREENFONTS;
                cf.lpLogFont = ptrLf;

                // The following way of allocating memory is not available in Silverlight,
                // so we use GCHandle.Alloc instead
                //IntPtr lpcf = Marshal.AllocHGlobal(Marshal.SizeOf(cf));
                cfBufferHandle = GCHandle.Alloc(new byte[Marshal.SizeOf(cf)], GCHandleType.Pinned);
                IntPtr ptrCf = cfBufferHandle.AddrOfPinnedObject();

                // copy the structure to the unmanaged memory
                Marshal.StructureToPtr(cf, ptrCf, false);

                // show the dialog!
                if (!ChooseFont(ptrCf))
                {
                    // user cancelled
                    return null;
                }

                // copy back: this method fails due to LogFontStruct being a value type.
                // there is a second overload of this method in .NET that would work, but
                // it's not accessible in Silverlight, so we manually copy over the byte array
                // values to the struct :-/
                //Marshal.PtrToStructure(cf.lpLogFont, lf);
                byte[] rawData = (byte[])lfBufferHandle.Target;
                lf = new LogFontStruct(rawData);

                // create the result
                var result = lf.ToFontData();
                return result;
            }
            finally
            {
                if (lfBufferHandle.IsAllocated)
                {
                    lfBufferHandle.Free();
                }

                if (cfBufferHandle.IsAllocated)
                {
                    cfBufferHandle.Free();
                }
            }
        }
    }
}
